"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createBranchDiff = createBranchDiff;
exports.deleteProject_SonarQube = deleteProject_SonarQube;
exports.getExemptionFileInfoPath = getExemptionFileInfoPath;
exports.getFilesSonarQube_API = getFilesSonarQube_API;
exports.getGitProjectId = getGitProjectId;
exports.isExemptionApplied = isExemptionApplied;
exports.requestAPI = requestAPI;
exports.requestUtilsCLI = requestUtilsCLI;
var _child_process = require("child_process");
var _fs = require("fs");
var _fileUtils = require("./fileUtils");
var _getSonarProperties = require("../core/sonar-dependencies/getSonarProperties");
var _hash = require("./hash");
var _logger = require("./logger");
var _http = _interopRequireDefault(require("http"));
var _path = _interopRequireDefault(require("path"));
var _querystring = _interopRequireDefault(require("querystring"));
/**
 * @function requestAPI - native request api 
 * @param {JSON} apiDetails 
 * @param {JSON} body 
 * @param {JSON} options 
 * @returns Promise<JSON>
 */

function requestAPI(apiDetails, body, options = {}) {
  return new Promise(resolve => {
    var request = _http.default.request(apiDetails, function (res) {
      var chunks = [];
      res.on("data", function (chunk) {
        chunks.push(chunk);
      });
      res.on("end", function (chunk) {
        var body = Buffer.concat(chunks);
        resolve(body.toString());
      });
      res.on("error", function (error) {
        _logger.Logger.log(_logger.Logger.FAILURE_TYPE, error);
      });
    });
    request.write(body);
    request.end();
  });
}

/**
 *@function requestUtilsCLI - method execute curl cli request
 * @returns {Promise<Buffer>}
 */

function requestUtilsCLI(command) {
  return (0, _child_process.execSync)(command, {
    shell: true
  });
}

/**
 * @function createBranchDiff - make giff diff api via curl api
 * @returns {void}
 */

function createBranchDiff() {
  var command;
  var {
    token,
    gitEndPoint,
    branchDiffPath,
    compareBranch,
    projectId
  } = (0, _fileUtils.getLintConfiguration)();
  if (projectId == "project-id") {
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `\n Make sure that project id is properly configured`);
    projectId = getGitProjectId();
  }
  const endPoint = `${gitEndPoint}/api/v4/projects/${projectId.toString()}/repository/compare?from=${compareBranch}&to=${(0, _getSonarProperties.getBranchName)()}`;
  try {
    command = `curl --header "PRIVATE-TOKEN:${(0, _hash.decrypt)(token, 12)}" "${endPoint}"`;
    (0, _fs.writeFileSync)(branchDiffPath, requestUtilsCLI(command));
  } catch (err) {
    _logger.Logger.log(_logger.Logger.INFO_TYPE, `\n INFO : If you are using a VPN and encounter an SSL certification issue, ensure that the proxy is enabled for SSH and shell connections.`);
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `\n Make sure that you have access to this repository ${(0, _getSonarProperties.getRepoName)()}`);
  }
}

/**
 * @function getGitProjectId -  retrives project id of repository
 * @returns string
 */

function getGitProjectId() {
  var command;
  const {
    token,
    gitEndPoint
  } = (0, _fileUtils.getLintConfiguration)();
  const endPoint = `${gitEndPoint}/api/v4/projects?search=${(0, _getSonarProperties.getRepoName)()}`;
  try {
    command = (0, _child_process.execSync)(`curl --header "PRIVATE-TOKEN:${(0, _hash.decrypt)(token, 12)}" "${endPoint}"`);
    return JSON.parse(command.toString()).pop().id;
  } catch (err) {
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `make sure that you have access to this repository ${(0, _getSonarProperties.getRepoName)()}`);
    _logger.Logger.log(_logger.Logger.INFO_TYPE, err);
  }
}

/**
 * @function getFilesSonarQube_API -  returns array of Paths after filteration of response of sonarQube issue api
 * @param {String} projectName  - project name of lint report
 * @returns Array<String>
 */

function getFilesSonarQube_API(projectName) {
  return new Promise(async resolve => {
    try {
      const {
        metricServerHost
      } = (0, _fileUtils.getLintConfiguration)();
      const encryptedSonarToken = "cmFqYXNla2FyLmhtOnJhamFzZWthci5obQ==";
      const sonarQubeIssueGetAPI = `${metricServerHost}/api/issues/search?componentKeys=${projectName}&s=FILE_LINE&resolved=false&ps=100&facets=severities,types&additionalFields=_all&timeZone=Asia/Calcutta`;
      var command = (0, _child_process.execSync)(`curl --location "${sonarQubeIssueGetAPI}" --header "Authorization: Basic ${encryptedSonarToken}"`);
      (0, _fs.writeFileSync)(getExemptionFileInfoPath(), JSON.stringify(command.toString()));
    } catch (error) {
      _logger.Logger.log(_logger.Logger.FAILURE_TYPE, error);
      return [];
    }
    const exemptionDetails = JSON.parse(require(getExemptionFileInfoPath()));
    const result = exemptionDetails !== null && exemptionDetails !== void 0 && exemptionDetails.issues ? Array.from(new Set(exemptionDetails.issues.map(file => {
      return file.component.split(":").pop();
    }))) : [];
    await deleteProject_SonarQube(projectName);
    resolve(result);
  });
}

/**
 * @function getExemptionFileInfoPath - returns path of exemption details
 * @returns string
 */

function getExemptionFileInfoPath() {
  return _path.default.resolve(__dirname, "..", "..", "exemptionInfo.json");
}

/**
 * @function isExemptionApplied - check whether given branch raised exemption or not
 * @returns boolean
 */
async function isExemptionApplied() {
  _logger.Logger.log(_logger.Logger.INFO_TYPE, '\n Checking if an exemption has been raised for this branch');
  const {
    exemptionInstanceHost
  } = (0, _fileUtils.getLintConfiguration)();
  try {
    const requestBody = JSON.stringify({
      branchName: (0, _getSonarProperties.getBranchName)()
    });
    const {
      hostname,
      port,
      pathname
    } = new URL(`${exemptionInstanceHost}/getApprovedTicket`);
    const apiInfo = {
      method: "GET",
      hostname: hostname,
      port: port,
      path: pathname,
      headers: {
        "Content-Type": "text/json",
        Authentication: "password",
        "Content-Length": Buffer.byteLength(requestBody)
      },
      maxRedirects: 20
    };
    var exemption_response = JSON.parse(await requestAPI(apiInfo, requestBody));
  } catch (error) {
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, 'Seems like given invalid url');
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, error.message);
    return false;
  }
  _logger.Logger.log(_logger.Logger.INFO_TYPE, exemption_response.status == "Approved" ? `\n Exemption is Applied to this ${(0, _getSonarProperties.getBranchName)()}` : `\n Exemption is not required for this ${(0, _getSonarProperties.getBranchName)()}`);
  return exemption_response.status == "free" ? false : exemption_response.status == "Approved" ? true : false;
}

/**
 * @function deleteProject_SonarQube - this function delete project created in given sonarQube instance
 * @param {string} projectName - project name to delete  
 * @returns {Promiseany>}
 */
async function deleteProject_SonarQube(projectName) {
  try {
    const {
      metricServerHost
    } = (0, _fileUtils.getLintConfiguration)();
    const encryptedSonarToken = "cmFqYXNla2FyLmhtOnJhamFzZWthci5obQ==";
    const {
      hostname
    } = new URL(metricServerHost);
    const requestBody = _querystring.default.stringify({
      project: projectName
    });
    const apiDetails = {
      method: "POST",
      hostname: hostname,
      path: "/api/projects/delete",
      headers: {
        'Content-Type': 'application/x-www-form-urlencoded',
        Authorization: `Basic ${encryptedSonarToken}`
      },
      maxRedirects: 20
    };
    return await requestAPI(apiDetails, requestBody);
  } catch (error) {
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, error);
  }
}